@@ -3,9 +3,8 @@ from functools import lru_cache
 from PySide6.QtGui import QImage
 
 from foundry.game.gfx.drawable import MASK_COLOR, bit_reverse
-from foundry.game.gfx.GraphicsSet import GraphicsSet
+from foundry.game.gfx.GraphicsSet import GraphicsSetProtocol
 from foundry.game.gfx.Palette import NESPalette, PaletteGroup
-from foundry.smb3parse.objects.object_set import CLOUDY_GRAPHICS_SET
 
 PIXEL_OFFSET = 8  # both bits describing the color of a pixel are in separate 8 byte chunks at the same index
 
@@ -26,7 +25,7 @@ class Tile:
         object_index: int,
         palette_group: PaletteGroup,
         palette_index: int,
-        graphics_set: GraphicsSet,
+        graphics_set: GraphicsSetProtocol,
         mirrored=False,
     ):
         start = object_index * Tile.SIZE
@@ -39,12 +38,7 @@ class Tile:
         self.pixels = bytearray()
         self.mask_pixels = bytearray()
 
-        self.data = graphics_set.data[start : start + Tile.SIZE]
-
-        if graphics_set.number == CLOUDY_GRAPHICS_SET:
-            self.background_color_index = 2
-        else:
-            self.background_color_index = 0
+        self.data = bytearray(bytes(graphics_set))[start : start + Tile.SIZE]
 
         if mirrored:
             self._mirror()
@@ -66,7 +60,7 @@ class Tile:
             color = self.palette[color_index]
 
             # add alpha values
-            if color_index == self.background_color_index:
+            if color_index == 0:
                 self.pixels.extend(MASK_COLOR)
             else:
                 self.pixels.extend(NESPalette[color].toTuple()[:3])
