@@ -1,10 +1,11 @@
-from typing import List, Tuple
+from typing import List, Optional, Tuple
 from warnings import warn
 
 from PySide6.QtCore import QRect, QSize
-from PySide6.QtGui import QImage, QPainter
+from PySide6.QtGui import QColor, QImage, QPainter, Qt
 
 from foundry.core.Position import Position, PositionProtocol
+from foundry.core.Size import Size, SizeProtocol
 from foundry.game.File import ROM
 from foundry.game.gfx.drawable.Block import Block, get_block
 from foundry.game.gfx.GraphicsSet import GraphicsSetProtocol
@@ -57,6 +58,8 @@ BLANK = -1
 SCREEN_HEIGHT = 15
 SCREEN_WIDTH = 16
 
+MASK_COLOR = [0xFF, 0x33, 0xFF]
+
 
 class LevelObject(GeneratorObject):
     def __init__(
@@ -73,14 +76,10 @@ class LevelObject(GeneratorObject):
         self.object_set = ObjectSet(object_set)
 
         self.graphics_set = graphics_set
+        self._position = Position(0, 0)
+        self._ignore_rendered_position = False
 
-        self.x_position = 0
-        self.y_position = 0
-
-        self.rendered_base_x = 0
-        self.rendered_base_y = 0
-
-        self.palette_group = palette_group
+        self.palette_group = tuple(tuple(c for c in pal) for pal in palette_group)
 
         self.index_in_level = index
         self.objects_ref = objects_ref
@@ -110,17 +109,17 @@ class LevelObject(GeneratorObject):
 
         # position relative to the start of the level (top)
         self.original_y = data[0] & 0b0001_1111
-        self.y_position = self.original_y
+        self.position.y = self.original_y
 
         # position relative to the start of the level (left)
         self.original_x = data[1]
-        self.x_position = self.original_x
+        self.position.x = self.original_x
 
         if self.vertical_level:
-            offset = (self.x_position // SCREEN_WIDTH) * SCREEN_HEIGHT
+            offset = (self.position.x // SCREEN_WIDTH) * SCREEN_HEIGHT
 
-            self.y_position += offset
-            self.x_position %= SCREEN_WIDTH
+            self.position.y += offset
+            self.position.x %= SCREEN_WIDTH
 
         # describes what object it is
         self._obj_index = 0x00
@@ -129,8 +128,6 @@ class LevelObject(GeneratorObject):
 
         object_data = self.object_set.get_definition_of(self.type)
 
-        self.width = object_data.bmp_width
-        self.height = object_data.bmp_height
         self.orientation = GeneratorType(object_data.orientation)
         self.ending = EndType(object_data.ending)
         self.name = object_data.description
@@ -208,12 +205,6 @@ class LevelObject(GeneratorObject):
         self._render()
 
     def _render(self):
-        self.rendered_base_x = base_x = self.x_position
-        self.rendered_base_y = base_y = self.y_position
-
-        self.rendered_width = new_width = self.width
-        self.rendered_height = new_height = self.height
-
         try:
             self.index_in_level = self.objects_ref.index(self)
         except ValueError:
@@ -223,15 +214,10 @@ class LevelObject(GeneratorObject):
         blocks_to_draw = []
 
         if self.orientation == GeneratorType.TO_THE_SKY:
-            base_x = self.x_position
-            base_y = SKY
+            for _ in range(self.position.y):
+                blocks_to_draw.extend(self.blocks[0 : self.scale.width])
 
-            for _ in range(self.y_position):
-                blocks_to_draw.extend(self.blocks[0 : self.width])
-
-            blocks_to_draw.extend(self.blocks[-self.width :])
-
-            new_height = self.y_position + (self.height - 1)
+            blocks_to_draw.extend(self.blocks[-self.scale.width :])
 
         elif self.orientation == GeneratorType.DESERT_PIPE_BOX:
             # segments are the horizontal sections, which are 8 blocks long
@@ -240,17 +226,12 @@ class LevelObject(GeneratorObject):
 
             is_pipe_box_type_b = self.obj_index // 0x10 == 4
 
-            rows_per_box = self.height
+            rows_per_box = self.scale.height
             lines_per_row = 4
 
-            segment_width = self.width
+            segment_width = self.scale.width
             segments = (self.length + 1) * 2
 
-            box_height = lines_per_row * rows_per_box
-
-            new_width = segments * segment_width
-            new_height = box_height
-
             for row_number in range(rows_per_box):
                 for line in range(lines_per_row):
                     if is_pipe_box_type_b and row_number > 0 and line == 0:
@@ -263,9 +244,6 @@ class LevelObject(GeneratorObject):
                     for segment_number in range(segments):
                         blocks_to_draw.extend(self.blocks[start:stop])
 
-            # draw another last row
-            new_height += 1
-
             if is_pipe_box_type_b:
                 # draw another open row
                 start = segment_width
@@ -279,11 +257,9 @@ class LevelObject(GeneratorObject):
                 blocks_to_draw.extend(self.blocks[start:stop])
 
             # every line repeats the last block again for some reason
-            for end_of_line in range(len(blocks_to_draw), 0, -new_width):
+            for end_of_line in range(len(blocks_to_draw), 0, -(self.rendered_size.width - 1)):
                 blocks_to_draw.insert(end_of_line, blocks_to_draw[end_of_line - 1])
 
-            new_width += 1
-
         elif self.orientation in [
             GeneratorType.DIAG_DOWN_LEFT,
             GeneratorType.DIAG_DOWN_RIGHT,
@@ -291,17 +267,11 @@ class LevelObject(GeneratorObject):
             GeneratorType.DIAG_WEIRD,
         ]:
             if self.ending == EndType.UNIFORM:
-                new_height = (self.length + 1) * self.height
-                new_width = (self.length + 1) * self.width
-
                 left = [BLANK]
                 right = [BLANK]
                 slopes = self.blocks
 
             elif self.ending == EndType.END_ON_TOP_OR_LEFT:
-                new_height = (self.length + 1) * self.height
-                new_width = (self.length + 1) * (self.width - 1)  # without fill block
-
                 if self.orientation in [GeneratorType.DIAG_DOWN_RIGHT, GeneratorType.DIAG_UP_RIGHT]:
                     fill_block = self.blocks[0:1]
                     slopes = self.blocks[1:]
@@ -323,9 +293,6 @@ class LevelObject(GeneratorObject):
                     left = fill_block
 
             elif self.ending == EndType.END_ON_BOTTOM_OR_RIGHT:
-                new_height = (self.length + 1) * self.height
-                new_width = (self.length + 1) * (self.width - 1)  # without fill block
-
                 fill_block = self.blocks[-1:]
                 slopes = self.blocks[0:-1]
 
@@ -339,16 +306,16 @@ class LevelObject(GeneratorObject):
 
             rows = []
 
-            if self.height > self.width:
-                slope_width = self.width
+            if self.scale.height > self.scale.width:
+                slope_width = self.scale.width
             else:
                 slope_width = len(slopes)
 
-            for y in range(new_height):
-                amount_right = (y // self.height) * slope_width
-                amount_left = new_width - slope_width - amount_right
+            for y in range(self.rendered_size.height):
+                amount_right = (y // self.scale.height) * slope_width
+                amount_left = self.rendered_size.width - slope_width - amount_right
 
-                offset = y % self.height
+                offset = y % self.scale.height
 
                 rows.append(amount_left * left + slopes[offset : offset + slope_width] + amount_right * right)
 
@@ -357,44 +324,21 @@ class LevelObject(GeneratorObject):
                     row.reverse()
 
             if self.orientation in [GeneratorType.DIAG_DOWN_RIGHT, GeneratorType.DIAG_UP_RIGHT]:
-                if not self.height > self.width:
+                if not self.scale.height > self.scale.width:
                     rows.reverse()
 
-            if self.orientation == GeneratorType.DIAG_DOWN_RIGHT and self.height > self.width:
+            if self.orientation == GeneratorType.DIAG_DOWN_RIGHT and self.scale.height > self.scale.width:
                 # special case for 60 degree platform wire down right
                 for row in rows:
                     row.reverse()
 
-            if self.orientation in [GeneratorType.DIAG_UP_RIGHT]:
-                base_y -= new_height - 1
-
-            if self.orientation in [GeneratorType.DIAG_DOWN_LEFT]:
-                base_x -= new_width - slope_width
-
             for row in rows:
                 blocks_to_draw.extend(row)
 
         elif self.orientation in [GeneratorType.PYRAMID_TO_GROUND, GeneratorType.PYRAMID_2]:
             # since pyramids grow horizontally in both directions when extending
             # we need to check for new ground every time it grows
-
-            base_x += 1  # set the new base_x to the tip of the pyramid
-
-            for y in range(base_y, self.ground_level):
-                new_height = y - base_y
-                new_width = 2 * new_height
-
-                bottom_row = QRect(base_x, y, new_width, 1)
-
-                if any(
-                    [
-                        bottom_row.intersects(obj.get_rect()) and y == obj.get_rect().top()
-                        for obj in self.objects_ref[0 : self.index_in_level]
-                    ]
-                ):
-                    break
-
-            base_x = base_x - (new_width // 2)
+            rendered_size = self.rendered_size
 
             blank = self.blocks[0]
             left_slope = self.blocks[1]
@@ -402,8 +346,8 @@ class LevelObject(GeneratorObject):
             right_fill = self.blocks[3]
             right_slope = self.blocks[4]
 
-            for y in range(new_height):
-                blank_blocks = (new_width // 2) - (y + 1)
+            for y in range(rendered_size.height):
+                blank_blocks = (rendered_size.width // 2) - (y + 1)
                 middle_blocks = y  # times two
 
                 blocks_to_draw.extend(blank_blocks * [blank])
@@ -416,14 +360,11 @@ class LevelObject(GeneratorObject):
 
         elif self.orientation == GeneratorType.ENDING:
             page_width = 16
-            page_limit = page_width - self.x_position % page_width
-
-            new_width = page_width + page_limit + 1
-            new_height = (GROUND - 1) - SKY
+            page_limit = page_width - self.position.x % page_width
 
             for y in range(SKY, GROUND - 1):
                 blocks_to_draw.append(self.blocks[0])
-                blocks_to_draw.extend([self.blocks[1]] * (new_width - 1))
+                blocks_to_draw.extend([self.blocks[1]] * (self.rendered_size.width))
 
             # todo magic number
             # ending graphics
@@ -440,194 +381,120 @@ class LevelObject(GeneratorObject):
                 for x in range(page_width):
                     block_index = rom.get_byte(rom_offset + y * page_width + x - 1)
 
-                    block_position = (y_offset + y) * new_width + x + page_limit + 1
+                    block_position = (y_offset + y) * (self.rendered_size.width + 1) + x + page_limit + 1
                     blocks_to_draw[block_position] = block_index
 
             # the ending object is seemingly always 1 block too wide (going into the next screen)
-            for end_of_line in range(len(blocks_to_draw) - 1, 0, -new_width):
+            for end_of_line in range(len(blocks_to_draw) - 1, 0, -(self.rendered_size.width + 1)):
                 del blocks_to_draw[end_of_line]
 
-            new_width -= 1
-
-            # Mushroom/Fire flower/Star is categorized as an enemy
-
         elif self.orientation == GeneratorType.VERTICAL:
-            new_height = self.length + 1
-            new_width = self.width
-
             if self.ending == EndType.UNIFORM:
-                if self.is_4byte:
-                    # there is one VERTICAL 4-byte object: Vertically oriented X-blocks
-                    # the width is the primary expansion
-                    new_width = (self.obj_index & 0x0F) + 1
-
-                for _ in range(new_height):
-                    for y in range(self.height):
-                        for x in range(new_width):
-                            blocks_to_draw.append(self.blocks[y * self.height + x % self.width])
-
-                # adjust height for giant blocks, so that the rect is correct
-                new_height *= self.height
+                for _ in range(self.length + 1):
+                    for y in range(self.scale.height):
+                        for x in range(self.rendered_size.width):
+                            blocks_to_draw.append(self.blocks[y * self.scale.height + x % self.scale.width])
 
             elif self.ending == EndType.END_ON_TOP_OR_LEFT:
                 # in case the drawn object is smaller than its actual size
-                for y in range(min(self.height, new_height)):
-                    offset = y * self.width
-                    blocks_to_draw.extend(self.blocks[offset : offset + self.width])
+                for y in range(min(self.scale.height, self.rendered_size.height)):
+                    offset = y * self.scale.width
+                    blocks_to_draw.extend(self.blocks[offset : offset + self.scale.width])
 
-                additional_rows = new_height - self.height
+                additional_rows = self.rendered_size.height - self.scale.height
 
                 # assume only the last row needs to repeat
                 # todo true for giant blocks?
                 if additional_rows > 0:
-                    last_row = self.blocks[-self.width :]
+                    last_row = self.blocks[-self.scale.width :]
 
                     for _ in range(additional_rows):
                         blocks_to_draw.extend(last_row)
 
             elif self.ending == EndType.END_ON_BOTTOM_OR_RIGHT:
-                additional_rows = new_height - self.height
+                additional_rows = self.rendered_size.height - self.scale.height
 
                 # assume only the first row needs to repeat
                 # todo true for giant blocks?
                 if additional_rows > 0:
-                    last_row = self.blocks[0 : self.width]
+                    last_row = self.blocks[0 : self.scale.width]
 
                     for _ in range(additional_rows):
                         blocks_to_draw.extend(last_row)
 
                 # in case the drawn object is smaller than its actual size
-                for y in range(min(self.height, new_height)):
-                    offset = y * self.width
-                    blocks_to_draw.extend(self.blocks[offset : offset + self.width])
+                for y in range(min(self.scale.height, self.rendered_size.height)):
+                    offset = y * self.scale.width
+                    blocks_to_draw.extend(self.blocks[offset : offset + self.scale.width])
 
             elif self.ending == EndType.TWO_ENDS:
                 # object exists on ships
-                top_row = self.blocks[0 : self.width]
-                bottom_row = self.blocks[-self.width :]
+                top_row = self.blocks[0 : self.scale.width]
+                bottom_row = self.blocks[-self.scale.width :]
 
                 blocks_to_draw.extend(top_row)
 
-                additional_rows = new_height - 2
+                additional_rows = self.rendered_size.height - 2
 
                 # repeat second to last row
                 if additional_rows > 0:
                     for _ in range(additional_rows):
-                        blocks_to_draw.extend(self.blocks[-2 * self.width : -self.width])
+                        blocks_to_draw.extend(self.blocks[-2 * self.scale.width : -self.scale.width])
 
-                if new_height > 1:
+                if self.rendered_size.height > 1:
                     blocks_to_draw.extend(bottom_row)
 
         elif self.orientation in [GeneratorType.HORIZONTAL, GeneratorType.HORIZ_TO_GROUND, GeneratorType.HORIZONTAL_2]:
-            new_width = self.length + 1
-
-            downwards_extending_vine = 1, 0, 0x06
-            wooden_sky_pole = 4, 0, 0x04
-
-            if self.object_info in [downwards_extending_vine, wooden_sky_pole]:
-                new_width -= 1
-
-            if self.orientation == GeneratorType.HORIZ_TO_GROUND:
-                # to the ground only, until it hits something
-                for y in range(base_y, self.ground_level):
-                    bottom_row = QRect(base_x, y, new_width, 1)
-
-                    if any(
-                        [
-                            bottom_row.intersects(obj.get_rect()) and y == obj.get_rect().top()
-                            for obj in self.objects_ref[0 : self.index_in_level]
-                        ]
-                    ):
-                        new_height = y - base_y
-                        break
-                else:
-                    # nothing underneath this object, extend to the ground
-                    new_height = self.ground_level - base_y
-
-                if self.is_single_block:
-                    new_width = self.length
-
-                min_height = min(self.height, 2)
-
-                new_height = max(min_height, new_height)
-
-            elif self.orientation == GeneratorType.HORIZONTAL_2 and self.ending == EndType.TWO_ENDS:
-                # floating platforms seem to just be one shorter for some reason
-                new_width -= 1
-            else:
-                new_height = self.height + self.secondary_length
-
             if self.ending == EndType.UNIFORM and not self.is_4byte:
-                for y in range(new_height):
-                    offset = (y % self.height) * self.width
-
-                    for _ in range(0, new_width):
-                        blocks_to_draw.extend(self.blocks[offset : offset + self.width])
-
-                # in case of giant blocks
-                new_width *= self.width
+                for y in range(self.rendered_size.height):
+                    blocks_to_draw.extend(
+                        self.blocks[y * self.scale.width : (y + 1) * self.scale.width] * (self.length + 1)
+                    )
 
             elif self.ending == EndType.UNIFORM and self.is_4byte:
                 # 4 byte objects
                 top = self.blocks[0:1]
                 bottom = self.blocks[-1:]
 
-                new_height = self.height + self.secondary_length
-
-                # ceilings are one shorter than normal
-                if self.height > self.width:
-                    new_height -= 1
-
                 if self.orientation == GeneratorType.HORIZONTAL_2:
-                    for _ in range(0, new_height - 1):
-                        blocks_to_draw.extend(new_width * top)
+                    for _ in range(0, self.rendered_size.height - 1):
+                        blocks_to_draw.extend(self.rendered_size.width * top)
 
-                    blocks_to_draw.extend(new_width * bottom)
+                    blocks_to_draw.extend(self.rendered_size.width * bottom)
                 else:
-                    blocks_to_draw.extend(new_width * top)
+                    blocks_to_draw.extend(self.rendered_size.width * top)
 
-                    for _ in range(1, new_height):
-                        blocks_to_draw.extend(new_width * bottom)
+                    for _ in range(1, self.rendered_size.height):
+                        blocks_to_draw.extend(self.rendered_size.width * bottom)
 
             elif self.ending == EndType.END_ON_TOP_OR_LEFT:
-                for y in range(new_height):
-                    offset = y * self.width
+                for y in range(self.rendered_size.height):
+                    offset = y * self.scale.width
 
                     blocks_to_draw.append(self.blocks[offset])
 
-                    for x in range(1, new_width):
+                    for x in range(1, self.rendered_size.width):
                         blocks_to_draw.append(self.blocks[offset + 1])
 
             elif self.ending == EndType.END_ON_BOTTOM_OR_RIGHT:
-                for y in range(new_height):
-                    offset = y * self.width
+                for y in range(self.rendered_size.height):
+                    offset = y * self.scale.width
 
-                    for x in range(new_width - 1):
+                    for x in range(self.rendered_size.width - 1):
                         blocks_to_draw.append(self.blocks[offset])
 
-                    blocks_to_draw.append(self.blocks[offset + self.width - 1])
+                    blocks_to_draw.append(self.blocks[offset + self.scale.width - 1])
 
             elif self.ending == EndType.TWO_ENDS:
-                if self.orientation == GeneratorType.HORIZONTAL and self.is_4byte:
-                    # flat ground objects have an artificial limit of 2 lines
-                    if (
-                        self.object_set.number == PLAINS_OBJECT_SET
-                        and self.domain == 0
-                        and self.obj_index in range(0xC0, 0xE0)
-                    ):
-                        self.height = new_height = min(2, self.secondary_length + 1)
-                    else:
-                        new_height = self.secondary_length + 1
-
-                if self.width > len(self.blocks):
+                if self.scale.width > len(self.blocks):
                     raise ValueError(f"{self} does not provide enough blocks to fill a row.")
                 else:
                     start = 0
-                    end = self.width
+                    end = self.scale.width
 
-                for y in range(self.height):
-                    new_start = y * self.width
-                    new_end = (y + 1) * self.width
+                for y in range(self.scale.height):
+                    new_start = y * self.scale.width
+                    new_end = (y + 1) * self.scale.width
 
                     if new_end > len(self.blocks):
                         # repeat the last line of blocks to fill the object
@@ -639,13 +506,13 @@ class LevelObject(GeneratorObject):
                     left, *middle, right = self.blocks[start:end]
 
                     blocks_to_draw.append(left)
-                    blocks_to_draw.extend(middle * (new_width - 2))
+                    blocks_to_draw.extend(middle * (self.rendered_size.width - 2))
                     blocks_to_draw.append(right)
 
-                if not len(blocks_to_draw) % self.height == 0:
+                if not len(blocks_to_draw) % self.scale.height == 0:
                     warn(f"Blocks to draw are not divisible by height. {self}", RuntimeWarning)
 
-                new_width = int(len(blocks_to_draw) / self.height)
+                new_width = int(len(blocks_to_draw) / self.scale.height)
 
                 top_row = blocks_to_draw[0:new_width]
                 middle_blocks = blocks_to_draw[new_width : new_width * 2]
@@ -653,10 +520,10 @@ class LevelObject(GeneratorObject):
 
                 blocks_to_draw = top_row
 
-                for y in range(1, new_height - 1):
+                for y in range(1, self.rendered_size.height - 1):
                     blocks_to_draw.extend(middle_blocks)
 
-                if new_height > 1:
+                if self.rendered_size.height > 1:
                     blocks_to_draw.extend(bottom_row)
         else:
             if not self.orientation == GeneratorType.SINGLE_BLOCK_OBJECT:
@@ -664,12 +531,6 @@ class LevelObject(GeneratorObject):
                 # breakpoint()
 
             if self.name.lower() == "black boss room background":
-                new_width = SCREEN_WIDTH
-                new_height = SCREEN_HEIGHT
-
-                base_x = self.x_position // SCREEN_WIDTH * SCREEN_WIDTH
-                base_y = 0
-
                 blocks_to_draw = SCREEN_WIDTH * SCREEN_HEIGHT * [self.blocks[0]]
 
         # for not yet implemented objects and single block objects
@@ -678,47 +539,35 @@ class LevelObject(GeneratorObject):
         else:
             self.rendered_blocks = self.blocks
 
-        self.rendered_width = new_width
-        self.rendered_height = new_height
-        self.rendered_base_x = base_x
-        self.rendered_base_y = base_y
+        self.rect = QRect(
+            self.rendered_position.x, self.rendered_position.y, self.rendered_size.width, self.rendered_size.height
+        )
 
-        if new_width and not self.rendered_height == len(self.rendered_blocks) / new_width:
-            warn(
-                f"Not enough Blocks for calculated height: {self.name}. "
-                f"Blocks for height: {len(self.rendered_blocks) / new_width}. Rendered height: {self.rendered_height}",
-                RuntimeWarning,
-            )
+    def draw(self, painter: QPainter, block_length, transparent, blocks: Optional[list[Block]] = None):
+        size = self.rendered_size
+        size.width = max(size.width, 1)
 
-            self.rendered_height = len(self.rendered_blocks) / new_width
-        elif new_width == 0:
-            warn(
-                f"Calculated Width is 0, setting to 1: {self.name}. "
-                f"Blocks to draw: {len(self.rendered_blocks)}. Rendered height: {self.rendered_height}",
-                RuntimeWarning,
-            )
-
-            self.rendered_width = 1
-
-        self.rect = QRect(self.rendered_base_x, self.rendered_base_y, self.rendered_width, self.rendered_height)
-
-    def draw(self, painter: QPainter, block_length, transparent):
         for index, block_index in enumerate(self.rendered_blocks):
             if block_index == BLANK:
                 continue
 
-            x = self.rendered_base_x + index % self.rendered_width
-            y = self.rendered_base_y + index // self.rendered_width
+            x = self.rendered_position.x + index % size.width
+            y = self.rendered_position.y + index // size.width
 
-            self._draw_block(painter, block_index, x, y, block_length, transparent)
+            self._draw_block(painter, block_index, x, y, block_length, transparent, blocks=blocks)
 
-    def _draw_block(self, painter: QPainter, block_index, x, y, block_length, transparent):
-        block = get_block(
-            block_index,
-            tuple(tuple(c for c in pal) for pal in self.palette_group),
-            self.graphics_set,
-            bytes(self.tsa_data),
-        )
+    def _draw_block(
+        self, painter: QPainter, block_index, x, y, block_length, transparent, blocks: Optional[list[Block]] = None
+    ):
+        if blocks is not None:
+            block = blocks[block_index if block_index <= 0xFF else ROM().get_byte(block_index)]
+        else:
+            block = get_block(
+                block_index,
+                self.palette_group,
+                self.graphics_set,
+                bytes(self.tsa_data),
+            )
 
         block.draw(
             painter,
@@ -730,14 +579,11 @@ class LevelObject(GeneratorObject):
         )
 
     def move_by(self, dx: int, dy: int):
-        new_x = self.rendered_base_x + dx
-        new_y = self.rendered_base_y + dy
-
-        self.position = Position(new_x, new_y)
+        self.position = Position(self.position.x + dx, self.position.y + dy)
 
     @property
     def position(self) -> PositionProtocol:
-        return Position(self.x_position, self.y_position)
+        return self._position
 
     @position.setter
     def position(self, position: PositionProtocol) -> None:
@@ -747,21 +593,146 @@ class LevelObject(GeneratorObject):
         x = max(0, x)
 
         if self.orientation == GeneratorType.TO_THE_SKY:
-            y = self.rendered_base_y + y
+            y = self.rendered_position.y + y
         else:
             y = max(0, y)
 
-        x_diff = self.x_position - self.rendered_base_x
-        y_diff = self.y_position - self.rendered_base_y
-
-        self.rendered_base_x = int(x)
-        self.rendered_base_y = int(y)
-
-        self.x_position = self.rendered_base_x + x_diff
-        self.y_position = self.rendered_base_y + y_diff
-
+        self._position = position
         self._render()
 
+    @property
+    def rendered_position(self) -> PositionProtocol:
+        if self._ignore_rendered_position:
+            return Position(0, 0)
+        elif self.orientation == GeneratorType.TO_THE_SKY:
+            return Position(self.position.x, SKY)
+        elif self.orientation in [GeneratorType.DIAG_UP_RIGHT]:
+            return Position(self.position.x, self.position.y - self.rendered_size.height + 1)
+        elif self.orientation in [GeneratorType.DIAG_DOWN_LEFT]:
+            if self.object_set.number == 3 or self.object_set.number == 14:  # Sky or Hilly tileset
+                return Position(self.position.x - (self.rendered_size.width - self.scale.width + 1), self.position.y)
+            else:
+                return Position(self.position.x - (self.rendered_size.width - self.scale.width), self.position.y)
+
+        elif self.orientation in [GeneratorType.PYRAMID_TO_GROUND, GeneratorType.PYRAMID_2]:
+            return Position(self.position.x - (self.rendered_size.width // 2) + 1, self.position.y)
+        elif self.name.lower() == "black boss room background":
+            return Position(self.position.x // SCREEN_WIDTH * SCREEN_WIDTH, 0)
+        return self.position
+
+    @property
+    def scale(self) -> SizeProtocol:
+        return Size(self.definition.bmp_width, self.definition.bmp_height)
+
+    @property
+    def rendered_size(self) -> SizeProtocol:
+        if self.orientation == GeneratorType.TO_THE_SKY:
+            return Size(self.scale.width, self.position.y + self.scale.height - 1)
+        elif self.orientation == GeneratorType.DESERT_PIPE_BOX:
+            segments = (self.length + 1) * 2
+            return Size(segments * self.scale.width + 1, 4 * self.scale.height)
+        elif self.orientation in [
+            GeneratorType.DIAG_DOWN_LEFT,
+            GeneratorType.DIAG_DOWN_RIGHT,
+            GeneratorType.DIAG_UP_RIGHT,
+            GeneratorType.DIAG_WEIRD,
+        ]:
+            if self.ending == EndType.UNIFORM:
+                return Size((self.length + 1) * self.scale.width, (self.length + 1) * self.scale.height)
+            elif self.ending == EndType.END_ON_TOP_OR_LEFT:
+                return Size((self.length + 1) * (self.scale.width - 1), (self.length + 1))
+            else:
+                return Size((self.length + 1) * (self.scale.width - 1), (self.length + 1) * self.scale.height)
+        elif self.orientation in [GeneratorType.PYRAMID_TO_GROUND, GeneratorType.PYRAMID_2]:
+            size = Size(1, 1)
+            for y in range(self.position.y, self.ground_level):
+                size = Size(2 * (y - self.position.y), (y - self.position.y))
+                bottom_row = QRect(self.position.x, y, size.width, 1)
+                if any(
+                    [
+                        bottom_row.intersects(obj.get_rect()) and y == obj.get_rect().top()
+                        for obj in self.objects_ref[0 : self.index_in_level]
+                    ]
+                ):
+                    break
+            return size
+        elif self.orientation == GeneratorType.ENDING:
+            page_width = 16
+            page_limit = page_width - self.position.x % page_width
+            return Size(page_width + page_limit, (GROUND - 1) - SKY)
+        elif self.orientation == GeneratorType.VERTICAL:
+            size = Size(self.scale.width, self.length + 1)
+
+            if self.ending == EndType.UNIFORM:
+                if self.is_4byte:
+                    # there is one VERTICAL 4-byte object: Vertically oriented X-blocks
+                    # the width is the primary expansion
+                    size.width = (self.obj_index & 0x0F) + 1
+
+                # adjust height for giant blocks, so that the rect is correct
+                size.height *= self.scale.height
+            return size
+        elif self.orientation in [GeneratorType.HORIZONTAL, GeneratorType.HORIZ_TO_GROUND, GeneratorType.HORIZONTAL_2]:
+            size = Size(self.length + 1, self.scale.height)
+
+            downwards_extending_vine = 1, 0, 0x06
+            wooden_sky_pole = 4, 0, 0x04
+
+            if self.object_info in [downwards_extending_vine, wooden_sky_pole]:
+                size.width -= 1
+            if self.orientation == GeneratorType.HORIZ_TO_GROUND:
+                # to the ground only, until it hits something
+                for y in range(self.position.y, self.ground_level):
+                    bottom_row = QRect(self.position.x, y, size.width, 1)
+
+                    if any(
+                        [
+                            bottom_row.intersects(obj.get_rect()) and y == obj.get_rect().top()
+                            for obj in self.objects_ref[0 : self.index_in_level]
+                        ]
+                    ):
+                        size.height = y - self.position.y
+                        break
+                else:
+                    # nothing underneath this object, extend to the ground
+                    size.height = self.ground_level - self.position.y
+
+                if self.is_single_block:
+                    size.width = self.length
+
+                size.height = max(min(self.scale.height, 2), size.height)
+
+            elif self.orientation == GeneratorType.HORIZONTAL_2 and self.ending == EndType.TWO_ENDS:
+                # floating platforms seem to just be one shorter for some reason
+                size.width -= 1
+            else:
+                size.height = self.scale.height + self.secondary_length
+
+            if self.ending == EndType.UNIFORM and not self.is_4byte:
+                size.width *= self.scale.width  # in case of giant blocks
+            elif self.ending == EndType.UNIFORM and self.is_4byte:
+                size.height = self.scale.height + self.secondary_length
+
+                # ceilings are one shorter than normal
+                if self.scale.height > self.scale.width:
+                    size.height -= 1
+
+            elif self.ending == EndType.TWO_ENDS:
+                if self.orientation == GeneratorType.HORIZONTAL and self.is_4byte:
+                    # flat ground objects have an artificial limit of 2 lines
+                    if (
+                        self.object_set.number == PLAINS_OBJECT_SET
+                        and self.domain == 0
+                        and self.obj_index in range(0xC0, 0xE0)
+                    ):
+                        size.height = min(2, self.secondary_length + 1)
+                    else:
+                        size.height = self.secondary_length + 1
+            return size
+        elif self.name.lower() == "black boss room background":
+            return Size(SCREEN_WIDTH, SCREEN_HEIGHT)
+        return self.scale
+
     def expands(self):
         expands = EXPANDS_NOT
 
@@ -816,7 +787,7 @@ class LevelObject(GeneratorObject):
             return
 
         if self.primary_expansion() == EXPANDS_HORIZ:
-            length = x - self.x_position
+            length = x - self.position.x
 
             length = max(0, length)
             length = min(length, 0x0F)
@@ -826,7 +797,7 @@ class LevelObject(GeneratorObject):
             self.obj_index = base_index + length
             self.data[2] = self.obj_index
         else:
-            length = x - self.x_position
+            length = x - self.position.x
             length = max(0, length)
             length = min(length, 0xFF)
 
@@ -844,7 +815,7 @@ class LevelObject(GeneratorObject):
             return
 
         if self.primary_expansion() == EXPANDS_VERT:
-            length = y - self.y_position
+            length = y - self.position.y
 
             length = max(0, length)
             length = min(length, 0x0F)
@@ -854,7 +825,7 @@ class LevelObject(GeneratorObject):
             self.obj_index = base_index + length
             self.data[2] = self.obj_index
         else:
-            length = y - self.y_position
+            length = y - self.position.y
             length = max(0, length)
             length = min(length, 0xFF)
 
@@ -869,10 +840,10 @@ class LevelObject(GeneratorObject):
 
     def resize_by(self, dx: int, dy: int):
         if dx:
-            self.resize_x(self.x_position + dx)
+            self.resize_x(self.position.x + dx)
 
         if dy:
-            self.resize_y(self.y_position + dy)
+            self.resize_y(self.position.y + dy)
 
     def increment_type(self):
         self.change_type(True)
@@ -914,34 +885,40 @@ class LevelObject(GeneratorObject):
 
     def get_status_info(self) -> List[tuple]:
         return [
-            ("x", self.rendered_base_x),
-            ("y", self.rendered_base_y),
-            ("Width", self.rendered_width),
-            ("Height", self.rendered_height),
+            ("x", self.rendered_position.x),
+            ("y", self.rendered_position.y),
+            ("Width", self.rendered_size.width),
+            ("Height", self.rendered_size.height),
             ("Orientation", ORIENTATION_TO_STR[self.orientation]),
             ("Ending", ENDING_STR[self.ending]),
         ]
 
     def display_size(self, zoom_factor: int = 1):
-        return QSize(self.rendered_width * Block.SIDE_LENGTH, self.rendered_height * Block.SIDE_LENGTH) * zoom_factor
+        return (
+            QSize(self.rendered_size.width * Block.SIDE_LENGTH, self.rendered_size.height * Block.SIDE_LENGTH)
+            * zoom_factor
+        )
 
     def as_image(self) -> QImage:
-        self.rendered_base_x = 0
-        self.rendered_base_y = 0
+        self._ignore_rendered_position = True
 
         image = QImage(
-            QSize(self.rendered_width * Block.SIDE_LENGTH, self.rendered_height * Block.SIDE_LENGTH),
+            QSize(self.rendered_size.width * Block.SIDE_LENGTH, self.rendered_size.height * Block.SIDE_LENGTH),
             QImage.Format_RGB888,
         )
 
-        bg_color = self.palette_group.background_color
+        bg_color = QColor(*MASK_COLOR).rgb()
 
         image.fill(bg_color)
+        mask = image.createMaskFromColor(QColor(*MASK_COLOR).rgb(), Qt.MaskOutColor)
+        image.setAlphaChannel(mask)
 
         painter = QPainter(image)
 
         self.draw(painter, Block.SIDE_LENGTH, True)
 
+        self._ignore_rendered_position = False
+
         return image
 
     def to_bytes(self) -> bytearray:
@@ -952,16 +929,16 @@ class LevelObject(GeneratorObject):
             # seems like you can't convert the coordinates 1:1
             # there seems to be ambiguity
 
-            offset = self.y_position // SCREEN_HEIGHT
+            offset = self.position.y // SCREEN_HEIGHT
 
-            x_position = self.x_position + offset * SCREEN_WIDTH
-            y_position = self.y_position % SCREEN_HEIGHT
+            x_position = self.position.x + offset * SCREEN_WIDTH
+            y_position = self.position.y % SCREEN_HEIGHT
         else:
-            x_position = self.x_position
-            y_position = self.y_position
+            x_position = self.position.x
+            y_position = self.position.y
 
         if self.orientation in [GeneratorType.PYRAMID_TO_GROUND, GeneratorType.PYRAMID_2]:
-            x_position = self.rendered_base_x - 1 + self.rendered_width // 2
+            x_position = self.rendered_position.x - 1 + self.rendered_size.width // 2
 
         data.append((self.domain << 5) | y_position)
         data.append(x_position)
@@ -979,7 +956,7 @@ class LevelObject(GeneratorObject):
         return data
 
     def __repr__(self) -> str:
-        return f"LevelObject {self.name} at {self.x_position}, {self.y_position}"
+        return f"LevelObject {self.name} at {self.position.x}, {self.position.y}"
 
     def __eq__(self, other):
         if not isinstance(other, LevelObject):
