@@ -42,6 +42,9 @@ def get_minimal_icon_object(level_object: T) -> T:
         obj = factory.from_properties(
             domain=level_object.domain, object_index=level_object.obj_index, x=0, y=0, length=None, index=0
         )
+
+    obj.palette_group = level_object.palette_group  # transfer old palette group
+
     if not isinstance(obj, LevelObject):
         return level_object
 
@@ -63,7 +66,7 @@ class ObjectIcon(QWidget, Generic[T]):
         self,
         parent: Optional[QWidget],
         item: T,
-        background_color: bool = True,
+        background_color: bool = False,
     ):
         super().__init__(parent)
 
@@ -99,7 +102,7 @@ class ObjectIcon(QWidget, Generic[T]):
         painter = QPainter(self)
 
         if self.background_color:
-            painter.fillRect(event.rect(), self.item.palette_group.background_color)
+            painter.fillRect(event.rect(), self.item.palette_group[0][0])
 
         scaled_image = get_minimal_icon_object(self.item).as_image().scaled(self.size(), aspectMode=Qt.KeepAspectRatio)
 
